@@ -37,8 +37,7 @@ fgsea <- function(pathways, stats, minSize = 1, maxSize = length(stats)-1, gseaP
     res
 }
 
-
-preparePathwaysAndStats <- function(pathways, stats, minSize, maxSize, gseaParam, scoreType){
+prepareStats <- function(stats, scoreType="std", gseaParam=1) {
     # Error if stats are non-finite
     if (any(!is.finite(stats))){
         stop("Not all stats values are finite numbers")
@@ -61,6 +60,22 @@ preparePathwaysAndStats <- function(pathways, stats, minSize, maxSize, gseaParam
     stats <- sort(stats, decreasing=TRUE)
     stats <- abs(stats) ^ gseaParam
 
+    # scaling weights to integer values for more stable calculations
+    # sum(stats) < 2**30 avoids integer overflows
+    scaleCoeff <- 2**30/sum(stats)
+    if (scaleCoeff >= 1) {
+        # scaling by an integer coefficient avoids information loss for integer input stats
+        scaleCoeff <- floor(scaleCoeff)
+    }
+
+    # TODO: special treatment for small non-zero values?
+    stats <- round(stats * scaleCoeff)
+    storage.mode(stats) <- "integer"
+    return(stats)
+}
+
+preparePathwaysAndStats <- function(pathways, stats, minSize, maxSize, gseaParam, scoreType){
+    stats <- prepareStats(stats, scoreType, gseaParam)
     res <- preparePathways(pathways, universe=names(stats), minSize, maxSize)
 
     res$stats <- stats
